import os
import pickle

import matplotlib.pyplot as plt
import numpy as np

from altius_l1pp_inflight.configuration import altius_config
from altius_l1pp_inflight.scene.generation.solar.hmi_files import SolarDataScenario

times = []


def moving_average(v, window_size):
    width = int(window_size / 2)
    i = width
    moving_averages = np.zeros_like(v)
    while i < len(v) - width + 1:
        this_window = v[i - width: i + width]

        window_average = sum(this_window) / window_size
        moving_averages[i] = window_average
        i += 1

    moving_averages[0:width] = moving_averages[width]
    moving_averages[-width:] = moving_averages[-width - 1]
    return moving_averages


config_filename = 'globalkey_8x8_grid.yaml'
config = altius_config.AltiusL1ppInflightConfig(config_filename)
home_dir = config.get_processed_solar_prnu_file_directory()
num_vert_scans = config.get_num_vertical_raster_scans()
num_horz_scans = config.get_num_horizontal_raster_scans()
raster_grid_str = str(num_horz_scans) + 'x' + str(num_vert_scans)
data_dir = os.path.join(home_dir, 'altius_observation', raster_grid_str,
                        SolarDataScenario.NO_SUNSPOTS.to_string_directory())

# with open(os.path.join(data_dir, 'stats_altius.pickle'), 'rb') as f:
#     stats_altius = pickle.load(f)

# plt.figure()
# altius_signal = stats_altius['mean_altius']
# plt.imshow(altius_signal)
# plt.title('Mean Central Region in ALTIUS Pixels (188 x 188)')
# plt.colorbar()
#
# plt.figure()
# altius_noise = np.sqrt(stats_altius['sample_variance_altius'])
# plt.imshow(altius_noise)
# plt.title('Standard deviation of ALTIUS Pixels of Central Region (188 x 188)')
# plt.colorbar()
#
# plt.figure()
# snr_altius = altius_signal / altius_noise
# snr_altius[snr_altius > 500] = np.nan
# plt.imshow(snr_altius)
# plt.title('Central Region SNR in ALTIUS Pixels (188 x 188)')
# plt.colorbar()


with open(os.path.join(data_dir, 'centering_error.pickle'), 'rb') as f:
    centering_error = pickle.load(f)

with open(os.path.join(data_dir, 'calibration_image_flat.pickle'), 'rb') as f:
    calibration_image, calibration_image_error = pickle.load(f)
    calibration_image[np.isclose(calibration_image, 0.0)] = np.nan

with open(os.path.join(data_dir, 'calibration_image_flat_ideal.pickle'), 'rb') as f:
    ideal_calibrated_image, ideal_calibrated_image_error = pickle.load(f)
    ideal_calibrated_image[np.isclose(ideal_calibrated_image, 0.0)] = np.nan

with open(os.path.join(data_dir, 'calibration_image_flat_corrected_center.pickle'), 'rb') as f:
    corrected_calibrated_image, corrected_calibrated_image_error = pickle.load(f)
    corrected_calibrated_image[np.isclose(corrected_calibrated_image, 0.0)] = np.nan

with open(os.path.join(data_dir, 'prnu_error_relative_to_ideal_flattened.pickle'), 'rb') as f:
    uncorrected_prnu_rel_error = pickle.load(f)

corrected_prnu_rel_error = (corrected_calibrated_image - ideal_calibrated_image) / ideal_calibrated_image

plt.figure()
r = 100 * corrected_calibrated_image_error[160, :] / corrected_calibrated_image[160, :]
plt.plot(r)
plt.title('Percent Error on Corrected Calibration Image, Row 160')
plt.plot([0, len(r)], [1.0, 1.0], 'r--')
plt.plot([0, len(r)], [-1.0, -1.0], 'r--')

# plt.figure()
# plt.plot(100*rel_error[160, :])
# plt.title('Percent Error on Uncorrected Calibration Image, Row 160')
# plt.plot([0, len(rel_error[160, :])], [1.0, 1.0], 'r--')
# plt.plot([0, len(rel_error[160, :])], [-1.0, -1.0], 'r--')
#
# plt.figure()
# plt.plot(100*rel_error_corr[160, :])
# plt.title('Percent Error on Corrected Calibration Image, Row 160')
# plt.plot([0, len(rel_error_corr[160, :])], [1.0, 1.0], 'r--')
# plt.plot([0, len(rel_error_corr[160, :])], [-1.0, -1.0], 'r--')

plt.figure()
plt.imshow(calibration_image[:-2, :], interpolation='none')
plt.title('Uncorrected Calibration Image')
plt.clim(0.98 * np.nanmean(calibration_image), 1.02 * np.nanmean(calibration_image))  # same scale
plt.colorbar()

plt.figure()
plt.imshow(corrected_calibrated_image[:-2, :], interpolation='none')
plt.title('Center-and-Radius-Corrected Calibration Image')
plt.clim(0.98 * np.nanmean(calibration_image), 1.02 * np.nanmean(calibration_image))  # same scale
plt.colorbar()

plt.figure()
plt.imshow(ideal_calibrated_image[:-2, :], interpolation='none')
plt.title('Ideal Calibrated Image')
plt.clim(0.98 * np.nanmean(calibration_image), 1.02 * np.nanmean(calibration_image))  # same scale
plt.colorbar()

r = corrected_calibrated_image[160, :]
plt.figure()
plt.plot(r, linewidth=0.8)
plt.plot([0, len(r)], [0.99 * np.nanmean(r), 0.99 * np.nanmean(r)], 'r--')
plt.plot([0, len(r)], [1.01 * np.nanmean(r), 1.01 * np.nanmean(r)], 'r--')
ma = moving_average(r, 10)
plt.plot(ma)
plt.title('Comparison of Row 160 in Corrected Calibrated Image with 1% Requirement')

plt.figure()
plt.plot(uncorrected_prnu_rel_error[400, :] * 100)
plt.plot(corrected_prnu_rel_error[400, :] * 100)
plt.legend(['Uncorrected', 'Corrected'])
plt.title('Comparison of Row 400 - Percent Error')
plt.xlabel('Horizontal Pixels')
plt.ylabel('Percent')

plt.figure()
plt.plot(uncorrected_prnu_rel_error[700, :] * 100)
plt.plot(corrected_prnu_rel_error[700, :] * 100)
plt.legend(['Uncorrected', 'Corrected'])
plt.title('Comparison of Row 700 Percent Error')
plt.xlabel('Horizontal Pixels')
plt.ylabel('Percent')

plt.figure()
plt.plot(uncorrected_prnu_rel_error[200, :] * 100)
plt.plot(corrected_prnu_rel_error[200, :] * 100)
plt.legend(['Uncorrected', 'Corrected'])
plt.title('Comparison of Row 200 Percent Error')
plt.xlabel('Horizontal Pixels')
plt.ylabel('Percent')

plt.figure()
plt.plot(uncorrected_prnu_rel_error[1500, :] * 100)
plt.plot(corrected_prnu_rel_error[1500, :] * 100)
plt.legend(['Uncorrected', 'Corrected'])
plt.title('Comparison of Row 1500 Percent Error')
plt.xlabel('Horizontal Pixels')
plt.ylabel('Percent')

plt.figure()
plt.imshow(uncorrected_prnu_rel_error[:-2, :])
plt.title('Uncorrected Relative Error')
plt.clim(np.nanmin(uncorrected_prnu_rel_error[:-2, :]), np.nanmax(uncorrected_prnu_rel_error[:-2, :]))
plt.colorbar()

plt.figure()
plt.imshow(corrected_prnu_rel_error[:-2, :])
plt.title('Center-and-Radius-Corrected Relative Error')
plt.clim(np.nanmin(uncorrected_prnu_rel_error[:-2, :]), np.nanmax(uncorrected_prnu_rel_error[:-2, :]))
plt.colorbar()

print('DeltaRadius   DeltaHoriz    DeltaVert')
for i in range(len(centering_error)):
    string_val = ''
    for j in range(len(centering_error[i])):
        string_val = string_val + '{:5f}'.format(centering_error[i, j]) + '    '
    print(string_val)
    print()

plt.show()
