import unittest
import numpy as np

import altius_l1pp_inflight.instrument.srf.gaussian_srf as g
import sasktran.solarspectrum as s


class TestGaussianSrf(unittest.TestCase):

    def test_uniform_gaussian_convolve(self):
        wl = np.arange(200, 400)
        srf = g.GaussianSrf(wl, 3.0 * np.ones_like(wl))

        sao = s.SolarSpectrum()

        wavelen = np.arange(300, 340, step=0.1)

        irrad_expected = sao.irradiance(wavelen, fwhm=3.0)

        hires_wl = np.arange(290, 350, step=0.001)
        hires_spectrum = sao.irradiance(hires_wl)
        irrad_actual = srf.convolve_spectrum_with_srf(wavelen, hires_wl, hires_spectrum).squeeze()

        rel_error = np.abs(irrad_actual - irrad_expected) / irrad_expected

        self.assertTrue((rel_error < 0.003).any(),
                        'Relative error between Gaussian SRF-convolved spectrum and gaussian-convolved SAO differ by >0.3%')
