import unittest

import matplotlib.pyplot as plt
import numpy as np

import altius_l1pp_inflight.scene.generation.lunar.rolo_wrapper as rolo


def print_simple_reflectance(show_plots):
    model = rolo.RoloModel()
    r1 = []
    r2 = []
    r3 = []
    phase = np.concatenate((np.linspace(-40, -1.5, 200), np.linspace(1.5, 40, 200)))
    for p in phase:
        r1.append(model.disk_reflectance(3, p, 0.0, 0.0, 7.0))
        r2.append(model.disk_reflectance(12, p, 0.0, 0.0, 7.0))
        r3.append(model.disk_reflectance(20, p, 0.0, 0.0, 7.0))

    w1 = model.wavelen_nm[3]
    w2 = model.wavelen_nm[12]
    w3 = model.wavelen_nm[20]

    if show_plots:
        plt.figure()
        plt.plot(phase, r1, phase, r2, phase, r3)
        plt.legend([str(w1) + ' nm', str(w2) + ' nm', str(w3) + ' nm'])
        plt.xlabel('Lunar Phase Angle')
        plt.ylabel('Reflectance')
        plt.title(r'ROLO Reflectance for $\theta = 0.0\degree, \phi = 0.0\degree, \Phi = 7.0\degree$')
        plt.show()

        r1 = []
        r2 = []
        r3 = []
        obs_lat = np.linspace(-8.0, 8.0, 100)
        for lat in obs_lat:
            r1.append(model.disk_reflectance(3, 1.5, lat, 3.5, 7.0))
            r2.append(model.disk_reflectance(3, 5.0, lat, 3.5, 7.0))
            r3.append(model.disk_reflectance(3, 10.0, lat, 3.5, 7.0))

        plt.figure()
        plt.plot(obs_lat, r1, obs_lat, r2, obs_lat, r3)
        plt.legend([r'$g = 1.5\degree$', r'$g = 5.0\degree$', r'$g = 10.0\degree$'])
        plt.xlabel(r'Sub-observer Selenographic Latitude, $\theta(\degree)$')
        plt.ylabel('Reflectance')
        plt.title(r'ROLO Reflectance for $\lambda = ' + str(w1) + r' nm, \phi = 3.5\degree, \Phi = 7.0\degree$')
        plt.show()


def print_reflectance_spectra(show_plots):
    model = rolo.RoloModel()
    r = []
    for i in range(0, 32):
        r.append(model.disk_reflectance(i, 7.0, 0.0, 0.0, 0.0))

    if show_plots:
        plt.figure()
        plt.plot(model.wavelen_nm, r, marker='.', linestyle='--')

        lunar_soil = model.lunar_soil()

        plt.plot(lunar_soil[:, 0], lunar_soil[:, 1])

        plt.legend(['ROLO', 'Lunar soil'])
        plt.xlabel('Wavelength (nm)')
        plt.ylabel('Reflectance')


class RoloReflectanceTest(unittest.TestCase):
    def test_simple_reflectance(self):
        print_simple_reflectance(False)

    def test_reflectance_spectra(self):
        print_reflectance_spectra(False)
