import matplotlib.pyplot as plt
import numpy as np

from altius_l1pp_inflight.configuration import altius_config
from altius_l1pp_inflight.scene.disk_geometry import SolarGeometry, D_1AU
from altius_l1pp_inflight.scene.generation.solar.hmi_files import HmiFileManager, SolarDataScenario
from altius_l1pp_inflight.scene.limbdark_mu_pierceandslaughter1977 import LimbDarkening_PS1977
from altius_l1pp_inflight.scene.sunspot_detection import detect_sunspots

config = altius_config.AltiusL1ppInflightConfig(test_config=True)
filemanager = HmiFileManager(config)

images = filemanager.load_scenario_files_as_images(SolarDataScenario.SEVERAL_SUNSPOTS)

# i0 = images[0].radiance_disk[2000:2800, 550:1400]
# i0[i0 < 1000] = np.nan
i0 = images[0].radiance_disk
X, Y = np.meshgrid(np.arange(0, 4096), np.arange(0, 4096))
R = np.sqrt((X - images[0]._h_c) ** 2 + (Y - images[0]._v_c) ** 2)

solar_geometry = SolarGeometry(4096, 4096, images[0]._v_c, images[0]._h_c, 0.6, 0.6)
mu = solar_geometry.set_disk(images[0]._average_radius_pixels, images[0]._h_c, images[0]._v_c, D_1AU)
mu_profile = np.linspace(0, 1, 50)
ld_profile = LimbDarkening_PS1977().limb_darkening(617.3, mu_profile)

ld = np.interp(mu, mu_profile, ld_profile)

i0_masked = np.copy(i0)
i0_masked /= ld

mask = (R - images[0]._average_radius_pixels) / images[0]._average_radius_pixels > -0.2
i0_masked[mask] = np.nan

plt.figure()
plt.imshow(i0)
plt.figure()
plt.imshow(i0_masked)

inverted_image, filtered_black2020, background_removed, sunspot_removed = detect_sunspots(i0_masked, 0.3)

sunspot_removed *= ld

plt.figure()
plt.plot(i0[2246, :])
plt.title('Original Cross-section with Sunspots')

i0[~mask] = np.copy(sunspot_removed[~mask])

plt.figure()
plt.plot(inverted_image[2246, :])
plt.title('Inverted Image')

plt.figure()
plt.plot(filtered_black2020[2246, :])
plt.title('Filter Applied to Inverted Image')

plt.figure()
plt.plot(background_removed[2246, :])
plt.title('Background Removed')

plt.figure()
plt.plot(i0[2246, :])
plt.title('Sunspots Removed')

plt.figure()
plt.imshow(i0)
plt.show()
